@@ -1,7 +1,7 @@
 package JSONY::Grammar;
 use base 'Pegex::Grammar';
 
-use constant file => 'share/jsony.pgx';
+use constant file => '../jsony-pgx/jsony.pgx';
 
 sub make_tree {
   {
@@ -11,17 +11,8 @@ sub make_tree {
     'EOL' => {
       '.rgx' => qr/\G\r?\n/
     },
-    'LCURLY' => {
-      '.rgx' => qr/\G\{/
-    },
-    'LSQUARE' => {
-      '.rgx' => qr/\G\[/
-    },
-    'RCURLY' => {
-      '.rgx' => qr/\G\}/
-    },
-    'RSQUARE' => {
-      '.rgx' => qr/\G\]/
+    '_' => {
+      '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
     },
     'bare' => {
       '.rgx' => qr/\G([^\s\{\}\[\]'",]*[^\s\{\}\[\]'",:])/
@@ -52,36 +43,40 @@ sub make_tree {
       ]
     },
     'list' => {
-      '+min' => 0,
-      '.ref' => 'node',
-      '.sep' => {
-        '+eok' => 1,
-        '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*,?(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
-      }
+      '+max' => 1,
+      '.all' => [
+        {
+          '.ref' => 'node'
+        },
+        {
+          '+min' => 0,
+          '-flat' => 1,
+          '.all' => [
+            {
+              '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*,?(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+            },
+            {
+              '.ref' => 'node'
+            }
+          ]
+        },
+        {
+          '+max' => 1,
+          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*,?(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+        }
+      ]
     },
     'map' => {
       '.all' => [
         {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
-        },
-        {
-          '.ref' => 'LCURLY'
-        },
-        {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*\{(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
         },
         {
           '+min' => 0,
           '.ref' => 'pair'
         },
         {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
-        },
-        {
-          '.ref' => 'RCURLY'
-        },
-        {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*\}(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
         }
       ]
     },
@@ -130,30 +125,34 @@ sub make_tree {
     'seq' => {
       '.all' => [
         {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*\[(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
         },
         {
-          '.ref' => 'LSQUARE'
+          '+max' => 1,
+          '.all' => [
+            {
+              '.ref' => 'node'
+            },
+            {
+              '+min' => 0,
+              '-flat' => 1,
+              '.all' => [
+                {
+                  '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*,?(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+                },
+                {
+                  '.ref' => 'node'
+                }
+              ]
+            },
+            {
+              '+max' => 1,
+              '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*,?(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+            }
+          ]
         },
         {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
-        },
-        {
-          '+min' => 0,
-          '.ref' => 'node',
-          '.sep' => {
-            '+eok' => 1,
-            '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*,?(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
-          }
-        },
-        {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
-        },
-        {
-          '.ref' => 'RSQUARE'
-        },
-        {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*\](?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
         }
       ]
     },
@@ -176,7 +175,7 @@ sub make_tree {
           '.ref' => 'node'
         },
         {
-          '.rgx' => qr/\G(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*/
+          '.ref' => '_'
         }
       ]
     },
@@ -192,12 +191,28 @@ sub make_tree {
         {
           '.all' => [
             {
-              '+min' => 0,
-              '.ref' => 'node',
-              '.sep' => {
-                '+eok' => 1,
-                '.rgx' => qr/\G(?:\ *,\ *\r?\n(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*|\ +)/
-              }
+              '+max' => 1,
+              '.all' => [
+                {
+                  '.ref' => 'node'
+                },
+                {
+                  '+min' => 0,
+                  '-flat' => 1,
+                  '.all' => [
+                    {
+                      '.rgx' => qr/\G(?:\ *,\ *\r?\n(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*|\ +)/
+                    },
+                    {
+                      '.ref' => 'node'
+                    }
+                  ]
+                },
+                {
+                  '+max' => 1,
+                  '.rgx' => qr/\G(?:\ *,\ *\r?\n(?:\s|(?:\#\ .*\r?\n|\#\r?\n|\ *\r?\n))*|\ +)/
+                }
+              ]
             },
             {
               '.any' => [
